/*
 * signals/catp.c --
 *
 *	This program demonstrates the POSIX library functions for
 *	handling signals. The POSIX API allows programs much more
 *	control over the behaviour of signals and signal handlers.
 */

#define _POSIX_C_SOURCE 200809L

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>

static void
handler(int signum)
{
#ifdef __APPLE__
    fprintf(stderr, "catp: handler: %s\n", strsignal(signum));
#else
    psignal(signum, "catp: handler");
#endif
}

int
main(void)
{
    char c;
    struct sigaction sa;

    sa.sa_handler = handler;
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = SA_RESTART;

    if (sigaction(SIGINT, &sa, NULL) == -1) {
        perror("sigaction");
        return EXIT_FAILURE;
    }

    while ((c = getc(stdin)) != EOF) {
        (void) putc(c, stdout);
    }

    if (ferror(stdin) || fflush(stdout) == EOF) {
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
